# RetDec IDA plugin

RetDec plugin for IDA (Interactive Disassembler).

The plugin is compatible with the following IDA versions: 6.6, 6.7, 6.8, 6.9, 6.95.
The plugin does NOT work with IDA 7.x.

## Installation and Use

Currently, we support only Windows and Linux.

### Windows

1. Either download and unpack a pre-built package from the following list, or build and install the RetDec IDA plugin by yourself (the process is described below):

    * [Windows - for 32-bit IDA](https://github.com/avast-tl/retdec-idaplugin/releases/download/v0.4/retdec-idaplugin-v0.4-win.zip) (v0.4)
    
2. Follow the user guide (`user_guide.pdf`) that is part of the downloaded package, or use the [current version](https://github.com/avast-tl/retdec-idaplugin/blob/master/doc/user_guide/user_guide.pdf) from this repository.

### Linux

1. Either download and unpack a pre-built package from the following list, or build and install the RetDec IDA plugin by yourself (the process is described below):

    * [Linux - for 32-bit IDA](https://github.com/avast-tl/retdec-idaplugin/releases/download/v0.4/retdec-idaplugin-v0.4-linux.zip) (v0.4)

2. Follow the user guide (`user_guide.pdf`) that is part of the downloaded package, or use the [current version](https://github.com/avast-tl/retdec-idaplugin/blob/master/doc/user_guide/user_guide.pdf) from this repository.

## Build and Installation

### Requirements

**Note: These are requirements to build the RetDec IDA plugin, not to run it. See our [User Guide](https://github.com/avast-tl/retdec-idaplugin/blob/master/doc/user_guide/user_guide.pdf) for information on plugin installation, configuration, and use.**

* A compiler supporting C++14
  * On Windows, only Microsoft Visual C++ is supported (version >= Visual Studio 2015).
* CMake (version >= 3.6)
* IDA SDK (version == 6.6)
* IDA plugin libraries
  * You can try to use prebuilt packages from [this repository](https://github.com/avast-tl/retdec-idaplugin-support).
  * Or you can build your own IDA plugin libraries with `scripts/download-and-build-libs.sh`. This requires the Linux bash shell. On Windows, you need to install and configure an environment like MSYS2. You can follow RetDec's Windows environment setup [guide](https://github.com/avast-tl/retdec/wiki/Windows-Environment) to help you get everything you need.
  * Or you can build IDA plugin libraries on your own.

### Process

* Recursively clone the repository (it contains submodules):
  * `git clone --recursive https://github.com/avast-tl/retdec-idaplugin.git`
* Linux:
  * `cd retdec-idaplugin`
  * `mkdir build && cd build`
  * `cmake .. -DIDAPLUGIN_LIBS_DIR=<path> -DIDA_SDK_DIR=<path>`
  * `make`
  * `make install` (if `IDA_PATH` was set, see below)
* Windows:
  * Open MSBuild command prompt, or any terminal that is configured to run the `msbuild` command.
  * `cd retdec-idaplugin`
  * `mkdir build && cd build`
  * `cmake .. -DIDAPLUGIN_LIBS_DIR=<path> -DIDA_SDK_DIR=<path> -G<generator>`
  * `msbuild /m /p:Configuration=Release retdec-idaplugin.sln`
  * `msbuild /m /p:Configuration=Release INSTALL.vcxproj` (if `IDA_PATH` was set, see below)
  * Alternatively, you can open `retdec-idaplugin.sln` generated by `cmake` in Visual Studio IDE.

You must pass the following parameters to `cmake`:
* `-DIDAPLUGIN_LIBS_DIR=</path/to/idaplugin/libs>` to tell `cmake` where the IDA plugin libraries are located.
* `-DIDA_SDK_DIR=</path/to/idasdk>` to tell `cmake` where the IDA SDK is located.
* (Windows only) `-G<generator>` is `-G"Visual Studio 14 2015"` for 32-bit build using Visual Studio 2015. Later versions of Visual Studio may be used. Only 32-bit build is supported.

You can pass the following additional parameters to `cmake`:
* `-DIDA_PATH=</path/to/ida>` to tell `cmake` where to install the plugin. If specified, installation will copy plugin binary into `IDA_PATH/plugins`, and content of `scripts/idc` directory into `IDA_PATH/idc`. If not set, installation step does nothing.

## User Guide

The [User Guide](https://github.com/avast-tl/retdec-idaplugin/blob/master/doc/user_guide/user_guide.pdf) in a PDF form is located in `doc/user_guide/user_guide.pdf`.

You can build your own guide by running `make` in `doc/user_guide` directory:
* Requires [GNU make](https://www.gnu.org/software/make/).
* Requires [LaTeX](https://www.latex-project.org/), LaTeX packages, and related tools.
* The resulting PDF will overwrite the original `user_guide.pdf`.

## License

Copyright (c) 2017 Avast Software, licensed under the MIT license. See the `LICENSE` file for more details.

RetDec IDA plugin uses third-party libraries or other resources listed, along with their licenses, in the `LICENSE-THIRD-PARTY` file.

## Contributing

See [RetDec contribution guidelines](https://github.com/avast-tl/retdec/wiki/Contribution-Guidelines).
